@@ -1,26 +1,26 @@
-""" 
+"""
  @file
  @brief This file contains the titles model, used by the title editor window
  @author Jonathan Thomas <jonathan@openshot.org>
- 
+
  @section LICENSE
- 
+
  Copyright (c) 2008-2018 OpenShot Studios, LLC
  (http://www.openshotstudios.com). This file is part of
  OpenShot Video Editor (http://www.openshot.org), an open-source project
  dedicated to delivering high quality video editing and animation solutions
  to the world.
- 
+
  OpenShot Video Editor is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
- 
+
  OpenShot Video Editor is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
@@ -28,8 +28,8 @@
 import os
 import fnmatch
 
-from PyQt5.QtCore import QMimeData, Qt, QSize
-from PyQt5.QtGui import *
+from PyQt5.QtCore import Qt, QObject, QMimeData
+from PyQt5.QtGui import QStandardItemModel, QStandardItem, QIcon
 from PyQt5.QtWidgets import QMessageBox
 import openshot  # Python module for libopenshot (required video editing module installed separately)
 
@@ -37,25 +37,23 @@ from classes import info
 from classes.logger import log
 from classes.app import get_app
 
-try:
-    import json
-except ImportError:
-    import simplejson as json
+import json
 
 
 class TitleStandardItemModel(QStandardItemModel):
-    def __init__(self, parent=None):
-        QStandardItemModel.__init__(self)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.setObjectName("titles.model")
 
     def mimeData(self, indexes):
         # Create MimeData for drag operation
         data = QMimeData()
 
         # Get list of all selected file ids
-        files = []
-        for item in indexes:
-            selected_row = self.itemFromIndex(item).row()
-            files.append(self.item(selected_row, 2).text())
+        files = [
+            self.itemFromIndex(i).data(TitleRoles.PathRole)
+            for i in indexes
+        ]
         data.setText(json.dumps(files))
         data.setHtml("title")
 
@@ -63,14 +61,15 @@ class TitleStandardItemModel(QStandardItemModel):
         return data
 
 
-class TitlesModel():
-    def update_model(self, clear=True):
-        log.info("updating title model.")
-        app = get_app()
+class TitleRoles:
+    PathRole = Qt.UserRole + 11
 
+
+class TitlesModel(QObject):
+    def update_model(self, clear=True):
+        log.debug("Updating title model")
         # Get window to check filters
-        win = app.window
-        _ = app._tr
+        _ = self.app._tr
 
         # Clear all items
         if clear:
@@ -78,28 +77,29 @@ class TitlesModel():
             self.model.clear()
 
         # Add Headers
-        self.model.setHorizontalHeaderLabels([_("Thumb"), _("Name")])
+        self.model.setHorizontalHeaderLabels([_("Name")])
 
-        # get a list of files in the OpenShot /transitions directory
+        # get a list of files in the OpenShot package directory
         titles_dir = os.path.join(info.PATH, "titles")
-
-        # Add build-in templates
-        titles_list = []
-        for filename in sorted(os.listdir(titles_dir)):
-            titles_list.append(os.path.join(titles_dir, filename))
-
+        titles_list = [
+            os.path.join(titles_dir, filename)
+            for filename in sorted(os.listdir(titles_dir))
+            ]
         # Add user-defined titles (if any)
-        for file in sorted(os.listdir(info.TITLE_PATH)):
-            # pretty up the filename for display purposes
-            if fnmatch.fnmatch(file, '*.svg'):
-                titles_list.append(os.path.join(info.TITLE_PATH, file))
+        titles_list.extend([
+            os.path.join(info.USER_TITLES_PATH, filename)
+            for filename in sorted(os.listdir(info.USER_TITLES_PATH))
+            if fnmatch.fnmatch(filename, '*.svg')
+            ])
 
         for path in sorted(titles_list):
-            (parent_path, filename) = os.path.split(path)
-            (fileBaseName, fileExtension) = os.path.splitext(filename)
+            filename = os.path.basename(path)
+            fileBaseName = os.path.splitext(filename)[0]
 
             # Skip hidden files (such as .DS_Store, etc...)
-            if filename[0] == "." or "thumbs.db" in filename.lower() or filename.lower() == "temp.svg":
+            if (filename[0] == "."
+               or "thumbs.db" in filename.lower()
+               or filename.lower() == "temp.svg"):
                 continue
 
             # split the name into parts (looking for a number)
@@ -108,7 +108,7 @@ class TitlesModel():
             if name_parts[-1].isdigit():
                 suffix_number = name_parts[-1]
 
-            # get name of transition
+            # get name of title template
             title_name = fileBaseName.replace("_", " ").capitalize()
 
             # replace suffix number with placeholder (if any)
@@ -119,74 +119,63 @@ class TitlesModel():
                 title_name = self.app._tr(title_name)
 
             # Check for thumbnail path (in build-in cache)
-            thumb_path = os.path.join(info.IMAGES_PATH, "cache",  "{}.png".format(fileBaseName))
-
-            # Check built-in cache (if not found)
+            thumb_path = os.path.join(info.IMAGES_PATH, "cache", "{}.png".format(fileBaseName))
             if not os.path.exists(thumb_path):
-                # Check user folder cache
+                # Check user folder cache (if not found)
                 thumb_path = os.path.join(info.CACHE_PATH, "{}.png".format(fileBaseName))
-
-            # Generate thumbnail (if needed)
             if not os.path.exists(thumb_path):
-
+                # Generate thumbnail (if needed)
                 try:
                     # Reload this reader
                     clip = openshot.Clip(path)
                     reader = clip.Reader()
-
-                    # Open reader
                     reader.Open()
 
-                    # Save thumbnail
-                    reader.GetFrame(0).Thumbnail(thumb_path, 98, 64, os.path.join(info.IMAGES_PATH, "mask.png"), "", "#000", True)
+                    # Save thumbnail to disk
+                    reader.GetFrame(0).Thumbnail(
+                        thumb_path, 98, 64,
+                        os.path.join(info.IMAGES_PATH, "mask.png"),
+                        "", "#000", True, "png", 85
+                    )
                     reader.Close()
                     clip.Close()
-
-                except:
-                    # Handle exception
+                except Exception as ex:
+                    log.info('Failed to open {} as title: {}'.format(filename, ex))
                     msg = QMessageBox()
-                    msg.setText(_("{} is not a valid image file.".format(filename)))
+                    msg.setText(_("%s is not a valid image file." % filename))
                     msg.exec_()
                     continue
-
-            row = []
-
-            # Append thumbnail
-            col = QStandardItem()
-            icon_pixmap = QPixmap(thumb_path)
-            scaled_pixmap = icon_pixmap.scaled(QSize(93, 62), Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
-            col.setIcon(QIcon(scaled_pixmap))
-            col.setText(title_name)
-            col.setToolTip(title_name)
-            col.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable | Qt.ItemIsDragEnabled)
-            row.append(col)
-
-            # Append Filename
-            col = QStandardItem("Name")
-            col.setData(title_name, Qt.DisplayRole)
-            col.setText(title_name)
-            col.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable | Qt.ItemIsDragEnabled)
-            row.append(col)
-
-            # Append Path
-            col = QStandardItem("Path")
-            col.setData(path, Qt.DisplayRole)
-            col.setText(path)
-            col.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable | Qt.ItemIsDragEnabled)
-            row.append(col)
-
-            # Append ROW to MODEL (if does not already exist in model)
-            if not path in self.model_paths:
-                self.model.appendRow(row)
+            # Create item entry for model
+            flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsUserCheckable | Qt.ItemIsDragEnabled
+            item = QStandardItem(QIcon(thumb_path), title_name)
+            item.setData(path, TitleRoles.PathRole)
+            item.setToolTip(title_name)
+            item.setFlags(flags)
+            # Append to MODEL (if does not already exist)
+            if path not in self.model_paths:
+                self.model.appendRow([item])
                 self.model_paths[path] = path
 
             # Process events in QT (to keep the interface responsive)
-            app.processEvents()
-
-    def __init__(self, *args):
+            self.app.processEvents()
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.setObjectName("TitlesModel")
         # Create standard model
         self.app = get_app()
-        self.model = TitleStandardItemModel()
-        self.model.setColumnCount(3)
+        self.model = TitleStandardItemModel(self.parent())
+        self.model.setColumnCount(1)
         self.model_paths = {}
+
+        # Attempt to load model testing interface, if requested
+        # (will only succeed with Qt 5.11+)
+        if info.MODEL_TEST:
+            try:
+                # Create model tester objects
+                from PyQt5.QtTest import QAbstractItemModelTester
+                QAbstractItemModelTester(
+                    self.model, QAbstractItemModelTester.FailureReportingMode.Warning)
+                log.info("Enabled model tests for title editor data")
+            except ImportError:
+                pass
