@@ -1,104 +1,113 @@
-""" 
+"""
  @file
  @brief This file contains the transitions file treeview, used by the main window
  @author Jonathan Thomas <jonathan@openshot.org>
- 
+
  @section LICENSE
- 
+
  Copyright (c) 2008-2018 OpenShot Studios, LLC
  (http://www.openshotstudios.com). This file is part of
  OpenShot Video Editor (http://www.openshot.org), an open-source project
  dedicated to delivering high quality video editing and animation solutions
  to the world.
- 
+
  OpenShot Video Editor is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
- 
+
  OpenShot Video Editor is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
 
-from PyQt5.QtCore import QSize, QPoint, Qt
-from PyQt5.QtGui import *
-from PyQt5.QtWidgets import QListView, QMenu
+from PyQt5.QtCore import Qt, QSize, QPoint, QRegExp
+from PyQt5.QtGui import QDrag
+from PyQt5.QtWidgets import QListView, QAbstractItemView, QMenu
 
 from classes.app import get_app
-from windows.models.transition_model import TransitionsModel
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
+from classes.logger import log
 
 
 class TransitionsListView(QListView):
     """ A QListView QWidget used on the main window """
-    drag_item_size = 48
+    drag_item_size = QSize(48, 48)
+    drag_item_center = QPoint(24, 24)
 
     def contextMenuEvent(self, event):
+        event.accept()
+
         # Set context menu mode
         app = get_app()
         app.context_menu_object = "transitions"
 
         menu = QMenu(self)
         menu.addAction(self.win.actionDetailsView)
-        menu.exec_(QCursor.pos())
+        menu.popup(event.globalPos())
 
-        # Ignore event, propagate to parent
-        event.ignore()
-
-    def startDrag(self, event):
+    def startDrag(self, supportedActions):
         """ Override startDrag method to display custom icon """
 
-        # Get image of selected item
-        selected_row = self.transition_model.model.itemFromIndex(self.selectionModel().selectedIndexes()[0]).row()
-        icon = self.transition_model.model.item(selected_row, 0).icon()
+        # Get first column indexes for all selected rows
+        selected = self.selectionModel().selectedRows(0)
+
+        # Get image of current item
+        current = self.selectionModel().currentIndex()
+        if not current.isValid() and selected:
+            current = selected[0]
+
+        if not current.isValid():
+            log.warning("No draggable items found in model!")
+            return False
+
+        # Get icon from column 0 on same row as current item
+        icon = current.sibling(current.row(), 0).data(Qt.DecorationRole)
 
         # Start drag operation
         drag = QDrag(self)
-        drag.setMimeData(self.transition_model.model.mimeData(self.selectionModel().selectedIndexes()))
-        # drag.setPixmap(QIcon.fromTheme('document-new').pixmap(QSize(self.drag_item_size,self.drag_item_size)))
-        drag.setPixmap(icon.pixmap(QSize(self.drag_item_size, self.drag_item_size)))
-        drag.setHotSpot(QPoint(self.drag_item_size / 2, self.drag_item_size / 2))
+        drag.setMimeData(self.model().mimeData(selected))
+        drag.setPixmap(icon.pixmap(self.drag_item_size))
+        drag.setHotSpot(self.drag_item_center)
         drag.exec_()
 
-    def clear_filter(self):
-        get_app().window.transitionsFilter.setText("")
-
     def filter_changed(self):
-        if self.win.transitionsFilter.text() == "":
-            self.win.actionTransitionsClear.setEnabled(False)
-        else:
-            self.win.actionTransitionsClear.setEnabled(True)
         self.refresh_view()
 
     def refresh_view(self):
-        self.transition_model.update_model()
+        """Filter transitions with proxy class"""
+        filter_text = self.win.transitionsFilter.text()
+        self.model().setFilterRegExp(QRegExp(filter_text.replace(' ', '.*')))
+        self.model().setFilterCaseSensitivity(Qt.CaseInsensitive)
+        self.model().sort(Qt.AscendingOrder)
 
-    def __init__(self, *args):
+    def __init__(self, model):
         # Invoke parent init
-        QListView.__init__(self, *args)
+        QListView.__init__(self)
 
         # Get a reference to the window object
         self.win = get_app().window
 
         # Get Model data
-        self.transition_model = TransitionsModel()
+        self.transition_model = model
 
         # Keep track of mouse press start position to determine when to start drag
         self.setAcceptDrops(True)
         self.setDragEnabled(True)
         self.setDropIndicatorShown(True)
 
+        self.setModel(self.transition_model.proxy_model)
+
+        # Remove the default selection model and wire up to the shared one
+        self.selectionModel().deleteLater()
+        self.setSelectionMode(QAbstractItemView.SingleSelection)
+        self.setSelectionBehavior(QAbstractItemView.SelectRows)
+        self.setSelectionModel(self.transition_model.selection_model)
+
         # Setup header columns
-        self.setModel(self.transition_model.model)
         self.setIconSize(QSize(131, 108))
         self.setGridSize(QSize(102, 92))
         self.setViewMode(QListView.IconMode)
@@ -108,10 +117,7 @@ class TransitionsListView(QListView):
         self.setTextElideMode(Qt.ElideRight)
         self.setStyleSheet('QListView::item { padding-top: 2px; }')
 
-        # Refresh view
-        self.refresh_view()
-
         # setup filter events
         app = get_app()
         app.window.transitionsFilter.textChanged.connect(self.filter_changed)
-        app.window.actionTransitionsClear.triggered.connect(self.clear_filter)
+        app.window.refreshTransitionsSignal.connect(self.refresh_view)
