@@ -1,42 +1,42 @@
-""" 
+"""
  @file
  @brief This file contains the transitions file treeview, used by the main window
  @author Jonathan Thomas <jonathan@openshot.org>
- 
+
  @section LICENSE
- 
+
  Copyright (c) 2008-2018 OpenShot Studios, LLC
  (http://www.openshotstudios.com). This file is part of
  OpenShot Video Editor (http://www.openshot.org), an open-source project
  dedicated to delivering high quality video editing and animation solutions
  to the world.
- 
+
  OpenShot Video Editor is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
- 
+
  OpenShot Video Editor is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public License
  along with OpenShot Library.  If not, see <http://www.gnu.org/licenses/>.
  """
 
-from PyQt5.QtCore import QSize, QPoint
-from PyQt5.QtGui import *
+from PyQt5.QtCore import Qt, QSize, QPoint
+from PyQt5.QtGui import QDrag
 from PyQt5.QtWidgets import QTreeView, QAbstractItemView, QMenu, QSizePolicy
 
 from classes.app import get_app
-from windows.models.transition_model import TransitionsModel
+from classes.logger import log
 
-import json
 
 class TransitionsTreeView(QTreeView):
     """ A TreeView QWidget used on the main window """
-    drag_item_size = 48
+    drag_item_size = QSize(48, 48)
+    drag_item_center = QPoint(24, 24)
 
     def contextMenuEvent(self, event):
         # Set context menu mode
@@ -45,59 +45,69 @@ class TransitionsTreeView(QTreeView):
 
         menu = QMenu(self)
         menu.addAction(self.win.actionThumbnailView)
-        menu.exec_(QCursor.pos())
+        menu.popup(event.globalPos())
 
     def startDrag(self, event):
         """ Override startDrag method to display custom icon """
 
-        # Get image of selected item
-        selected_row = self.transition_model.model.itemFromIndex(self.selectionModel().selectedIndexes()[0]).row()
-        icon = self.transition_model.model.item(selected_row, 0).icon()
+        # Get first column indexes for all selected rows
+        selected = self.selectionModel().selectedRows(0)
+
+        # Get image of current item
+        current = self.selectionModel().currentIndex()
+        if not current.isValid() and selected:
+            current = selected[0]
+
+        if not current.isValid():
+            log.warning("No draggable items found in model!")
+            return False
+
+        # Get icon from column 0 on same row as current item
+        icon = current.sibling(current.row(), 0).data(Qt.DecorationRole)
 
         # Start drag operation
         drag = QDrag(self)
-        drag.setMimeData(self.transition_model.model.mimeData(self.selectionModel().selectedIndexes()))
-        # drag.setPixmap(QIcon.fromTheme('document-new').pixmap(QSize(self.drag_item_size,self.drag_item_size)))
-        drag.setPixmap(icon.pixmap(QSize(self.drag_item_size, self.drag_item_size)))
-        drag.setHotSpot(QPoint(self.drag_item_size / 2, self.drag_item_size / 2))
+        drag.setMimeData(self.model().mimeData(selected))
+        drag.setPixmap(icon.pixmap(self.drag_item_size))
+        drag.setHotSpot(self.drag_item_center)
         drag.exec_()
 
-    def filter_changed(self):
-        self.refresh_view()
+    def refresh_columns(self):
+        """Hide certain columns"""
+        if type(self) == TransitionsTreeView:
+            # Only execute when the treeview is active
+            self.hideColumn(2)
+            self.hideColumn(3)
 
-    def refresh_view(self):
-        self.transition_model.update_model()
-        self.hideColumn(2)
-        self.hideColumn(3)
-
-    def __init__(self, *args):
+    def __init__(self, model):
         # Invoke parent init
-        QTreeView.__init__(self, *args)
+        QTreeView.__init__(self)
 
         # Get a reference to the window object
         self.win = get_app().window
 
         # Get Model data
-        self.transition_model = TransitionsModel()
+        self.transition_model = model
 
         # Keep track of mouse press start position to determine when to start drag
         self.setAcceptDrops(True)
         self.setDragEnabled(True)
         self.setDropIndicatorShown(True)
 
+        self.setModel(self.transition_model.proxy_model)
+
+        # Remove the default selection model and wire up to the shared one
+        self.selectionModel().deleteLater()
+        self.setSelectionMode(QAbstractItemView.SingleSelection)
+        self.setSelectionBehavior(QAbstractItemView.SelectRows)
+        self.setSelectionModel(self.transition_model.selection_model)
+
         # Setup header columns
-        self.setModel(self.transition_model.model)
         self.setIconSize(QSize(75, 62))
         self.setIndentation(0)
-        self.setSelectionBehavior(QTreeView.SelectRows)
-        self.setSelectionBehavior(QAbstractItemView.SelectRows)
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         self.setWordWrap(True)
         self.setStyleSheet('QTreeView::item { padding-top: 2px; }')
+        self.transition_model.ModelRefreshed.connect(self.refresh_columns)
 
-        # Refresh view
-        self.refresh_view()
-
-        # setup filter events
-        app = get_app()
-        app.window.transitionsFilter.textChanged.connect(self.filter_changed)
+        self.refresh_columns()
