@@ -27,14 +27,18 @@
 
 import functools
 
-from PyQt5.QtCore import Qt, QPoint, QRectF, QEvent
-from PyQt5.QtGui import *
-from PyQt5.QtWidgets import QLabel, QWidget, QDockWidget, QVBoxLayout, QHBoxLayout, QPushButton, QToolButton, QCheckBox
-
+from PyQt5.QtCore import Qt, QPoint, QRectF
+from PyQt5.QtGui import (
+    QColor, QPalette, QPen, QPainter, QPainterPath, QKeySequence,
+)
+from PyQt5.QtWidgets import (
+    QAction, QLabel, QWidget, QVBoxLayout, QHBoxLayout,
+    QPushButton, QToolButton, QCheckBox,
+)
 from classes.logger import log
-from classes.settings import get_settings
 from classes.app import get_app
-from classes.metrics import *
+from classes.metrics import track_metric_screen
+from classes import sentry
 
 
 class TutorialDialog(QWidget):
@@ -47,48 +51,51 @@ class TutorialDialog(QWidget):
         painter.setRenderHint(QPainter.Antialiasing)
         frameColor = QColor("#53a0ed")
 
-        # Paint blue rounded rectangle
-        path = QPainterPath()
-        path.addRoundedRect(QRectF(31, 0, self.width()-31, self.height()), 10, 10)
-        painter.setPen(Qt.NoPen)
-        painter.fillPath(path, frameColor)
-        painter.drawPath(path)
-
-        # Paint gray rounded rectangle
-        path = QPainterPath()
-        path.addRoundedRect(QRectF(32, 1, self.width()-33, self.height()-2), 10, 10)
-        painter.setPen(Qt.NoPen)
-        painter.fillPath(path, self.palette().color(QPalette.Window))
-        painter.drawPath(path)
+        painter.setPen(QPen(frameColor, 2))
+        painter.setBrush(self.palette().color(QPalette.Window))
+        painter.drawRoundedRect(
+            QRectF(31, 0,
+                   self.width() - 31,
+                   self.height()
+                   ),
+            10, 10)
 
         # Paint blue triangle (if needed)
         if self.arrow:
             arrow_height = 20
             path = QPainterPath()
-            path.moveTo (0, 35)
-            path.lineTo (31, 35 - arrow_height)
-            path.lineTo (31, (35 - arrow_height) + (arrow_height * 2))
-            path.lineTo (0, 35)
+            path.moveTo(0, 35)
+            path.lineTo(31, 35 - arrow_height)
+            path.lineTo(
+                31, int((35 - arrow_height) + (arrow_height * 2)))
+            path.lineTo(0, 35)
             painter.fillPath(path, frameColor)
-            painter.drawPath(path)
 
     def checkbox_metrics_callback(self, state):
         """ Callback for error and anonymous usage checkbox"""
-        s = get_settings()
+        s = get_app().get_settings()
         if state == Qt.Checked:
             # Enabling metrics sending
             s.set("send_metrics", True)
+            sentry.init_tracing()
 
             # Opt-in for metrics tracking
             track_metric_screen("metrics-opt-in")
         else:
             # Opt-out for metrics tracking
             track_metric_screen("metrics-opt-out")
+            sentry.disable_tracing()
 
             # Disable metric sending
             s.set("send_metrics", False)
 
-    def __init__(self, id, text, arrow, *args):
+    def mouseReleaseEvent(self, event):
+        """Process click events on tutorial. Especially useful when tutorial messages are partially
+        obscured or offscreen (i.e. on small screens). Just click any part of the tutorial, and it will
+        move on to the next one."""
+        self.manager.next_tip(self.widget_id)
+
+    def __init__(self, widget_id, text, arrow, manager, *args):
         # Invoke parent init
         QWidget.__init__(self, *args)
 
@@ -97,12 +104,13 @@ class TutorialDialog(QWidget):
         _ = app._tr
 
         # Keep track of widget to position next to
-        self.id = id
+        self.widget_id = widget_id
         self.arrow = arrow
+        self.manager = manager
 
         # Create vertical box
         vbox = QVBoxLayout()
-        vbox.setContentsMargins(32,10,10,10)
+        vbox.setContentsMargins(32, 10, 10, 10)
 
         # Add label
         self.label = QLabel(self)
@@ -110,14 +118,15 @@ class TutorialDialog(QWidget):
         self.label.setTextFormat(Qt.RichText)
         self.label.setWordWrap(True)
         self.label.setStyleSheet("margin-left: 20px;")
+        self.label.setAttribute(Qt.WA_TransparentForMouseEvents)
         vbox.addWidget(self.label)
 
         # Add error and anonymous metrics checkbox (for ID=0) tooltip
         # This is a bit of a hack, but since it's the only exception, it's
         # probably okay for now.
-        if self.id == "0":
+        if self.widget_id == "0":
             # Get settings
-            s = get_settings()
+            s = get_app().get_settings()
 
             # create spinner
             checkbox_metrics = QCheckBox()
@@ -132,29 +141,41 @@ class TutorialDialog(QWidget):
 
         # Add button box
         hbox = QHBoxLayout()
-        hbox.setContentsMargins(20,10,0,0)
+        hbox.setContentsMargins(20, 10, 0, 0)
+
+        # Close action
+        self.close_action = QAction(_("Hide Tutorial"), self)
+        self.close_action.setShortcut(QKeySequence(Qt.Key_Escape))
+        self.close_action.setShortcutContext(Qt.ApplicationShortcut)
 
         # Create buttons
         self.btn_close_tips = QPushButton(self)
         self.btn_close_tips.setText(_("Hide Tutorial"))
+        self.btn_close_tips.addAction(self.close_action)
+
         self.btn_next_tip = QPushButton(self)
         self.btn_next_tip.setText(_("Next"))
         self.btn_next_tip.setStyleSheet("font-weight:bold;")
+
         hbox.addWidget(self.btn_close_tips)
         hbox.addWidget(self.btn_next_tip)
         vbox.addLayout(hbox)
 
-        # Set layout
+        # Set layout, cursor, and size
         self.setLayout(vbox)
-
-        # Set size
+        self.setCursor(Qt.ArrowCursor)
         self.setMinimumWidth(350)
         self.setMinimumHeight(100)
+        self.setFocusPolicy(Qt.ClickFocus)
 
         # Make transparent
         self.setAttribute(Qt.WA_NoSystemBackground, True)
         self.setAttribute(Qt.WA_TranslucentBackground, True)
-        #self.setWindowFlags(Qt.FramelessWindowHint)
+        self.setAttribute(Qt.WA_DeleteOnClose, True)
+
+        # Connect close action signal
+        self.close_action.triggered.connect(
+            functools.partial(self.manager.hide_tips, self.widget_id, True))
 
 
 class TutorialManager(object):
@@ -163,9 +184,9 @@ class TutorialManager(object):
     def process(self, parent_name=None):
         """ Process and show the first non-completed tutorial """
 
-        # Do nothing if a tutorial is already visible
+        # If a tutorial is already visible, just update it
         if self.current_dialog:
-            # XXX: Respond to possible dock floats/moves
+            # Respond to possible dock floats/moves
             self.dock.raise_()
             self.re_position_dialog()
             return
@@ -174,29 +195,28 @@ class TutorialManager(object):
         for tutorial_details in self.tutorial_objects:
             # Get details
             tutorial_id = tutorial_details["id"]
-            tutorial_object_id = tutorial_details["object_id"]
-            tutorial_text = tutorial_details["text"]
-            tutorial_x_offset = tutorial_details["x"]
-            tutorial_y_offset = tutorial_details["y"]
-            turorial_arrow = tutorial_details["arrow"]
 
             # Get QWidget
-            tutorial_object = self.get_object(tutorial_object_id)
+            tutorial_object = self.get_object(tutorial_details["object_id"])
 
             # Skip completed tutorials (and invisible widgets)
-            if tutorial_object.visibleRegion().isEmpty() or tutorial_id in self.tutorial_ids or not self.tutorial_enabled:
+            if not self.tutorial_enabled or tutorial_id in self.tutorial_ids or tutorial_object.visibleRegion().isEmpty():
                 continue
 
             # Create tutorial
             self.position_widget = tutorial_object
-            self.x_offset = tutorial_x_offset
-            self.y_offset = tutorial_y_offset
-            tutorial_dialog = TutorialDialog(tutorial_id, tutorial_text, turorial_arrow)
+            self.offset = QPoint(
+                int(tutorial_details["x"]),
+                int(tutorial_details["y"]))
+            tutorial_dialog = TutorialDialog(tutorial_id, tutorial_details["text"], tutorial_details["arrow"], self)
 
             # Connect signals
             tutorial_dialog.btn_next_tip.clicked.connect(functools.partial(self.next_tip, tutorial_id))
             tutorial_dialog.btn_close_tips.clicked.connect(functools.partial(self.hide_tips, tutorial_id, True))
 
+            # Get previous dock contents
+            old_widget = self.dock.widget()
+
             # Insert into tutorial dock
             self.dock.setWidget(tutorial_dialog)
             self.current_dialog = tutorial_dialog
@@ -206,28 +226,39 @@ class TutorialManager(object):
             self.dock.setEnabled(True)
             self.re_position_dialog()
             self.dock.show()
+
+            # Delete old widget
+            if old_widget:
+                old_widget.close()
+
             break
 
     def get_object(self, object_id):
         """Get an object from the main window by object id"""
-        if object_id == "filesTreeView":
-            return self.win.filesTreeView
+        if object_id == "filesView":
+            return self.win.filesView
         elif object_id == "timeline":
             return self.win.timeline
-        elif object_id == "dockVideoContents":
-            return self.win.dockVideoContents
+        elif object_id == "dockVideo":
+            return self.win.dockVideo
         elif object_id == "propertyTableView":
             return self.win.propertyTableView
-        elif object_id == "transitionsTreeView":
-            return self.win.transitionsTreeView
-        elif object_id == "effectsTreeView":
-            return self.win.effectsTreeView
+        elif object_id == "transitionsView":
+            return self.win.transitionsView
+        elif object_id == "effectsView":
+            return self.win.effectsView
+        elif object_id == "emojisView":
+            return self.win.emojiListView
+        elif object_id == "actionPlay":
+            # Find play/pause button on transport controls toolbar
+            for w in self.win.actionPlay.associatedWidgets():
+                if isinstance(w, QToolButton):
+                    return w
         elif object_id == "export_button":
             # Find export toolbar button on main window
-            export_button = None
-            for toolbutton in self.win.toolBar.children():
-                if type(toolbutton) == QToolButton and toolbutton.defaultAction() and toolbutton.defaultAction().objectName() == "actionExportVideo":
-                    return toolbutton
+            for w in self.win.actionExportVideo.associatedWidgets():
+                if isinstance(w, QToolButton):
+                    return w
 
     def next_tip(self, tid):
         """ Mark the current tip completed, and show the next one """
@@ -239,7 +270,7 @@ class TutorialManager(object):
 
     def hide_tips(self, tid, user_clicked=False):
         """ Hide the current tip, and don't show anymore """
-        s = get_settings()
+        s = get_app().get_settings()
 
         # Loop through and find current tid
         for tutorial_object in self.tutorial_objects:
@@ -259,6 +290,9 @@ class TutorialManager(object):
             self.tutorial_enabled = False
             s.set("tutorial_enabled", False)
 
+        # Forgot current tutorial
+        self.current_dialog = None
+
     def close_dialogs(self):
         """ Close any open tutorial dialogs """
         if self.current_dialog:
@@ -274,9 +308,8 @@ class TutorialManager(object):
             self.win.dockEffects.visibilityChanged.disconnect()
             self.win.dockProperties.visibilityChanged.disconnect()
             self.win.dockVideo.visibilityChanged.disconnect()
-        except:
-            # Ignore errors from this
-            pass
+        except Exception:
+            log.debug('Failed to properly disconnect from dock signals', exc_info=1)
 
         # Close dialog window
         self.close_dialogs()
@@ -295,10 +328,24 @@ class TutorialManager(object):
     def re_position_dialog(self):
         """ Reposition a tutorial dialog next to another widget """
         if self.current_dialog:
-            """ Move widget next to its position widget """
-            x = self.position_widget.mapToGlobal(self.position_widget.pos()).x()
-            y = self.position_widget.mapToGlobal(self.position_widget.pos()).y()
-            self.dock.move(QPoint(x + self.x_offset, y + self.y_offset))
+            # Check if target is visible
+            if self.position_widget.isHidden():
+                self.hide_dialog()
+                return
+
+            # Locate tutorial popup relative to its "target" widget
+            pos_rect = self.position_widget.rect()
+
+            # Start with a 1/4-size offset rectangle, so the tutorial dialog
+            # floats a bit, then apply any custom offset defined for this popup.
+            pos_rect.setSize(pos_rect.size() / 4)
+            pos_rect.translate(self.offset)
+            # Map the new rectangle's bottom-right corner to global coords
+            position = self.position_widget.mapToGlobal(pos_rect.bottomRight())
+
+            # Move tutorial widget to the correct position
+            self.dock.move(position)
+            self.re_show_dialog()
 
     def __init__(self, win):
         """ Constructor """
@@ -311,35 +358,90 @@ class TutorialManager(object):
         _ = app._tr
 
         # get settings
-        s = get_settings()
+        s = app.get_settings()
         self.tutorial_enabled = s.get("tutorial_enabled")
         self.tutorial_ids = s.get("tutorial_ids").split(",")
 
         # Add all possible tutorials
-        self.tutorial_objects = [    {"id":"0", "x":400, "y":0, "object_id":"filesTreeView", "text":_("<b>Welcome!</b> OpenShot Video Editor is an award-winning, open-source video editing application! This tutorial will walk you through the basics.<br><br>Would you like to automatically send errors and metrics to help improve OpenShot?"), "arrow":False},
-                                     {"id":"1", "x":20, "y":0, "object_id":"filesTreeView", "text":_("<b>Project Files:</b> Get started with your project by adding video, audio, and image files here. Drag and drop files from your file system."), "arrow":True},
-                                     {"id":"2", "x":200, "y":-15, "object_id":"timeline", "text":_("<b>Timeline:</b> Arrange your clips on the timeline here. Overlap clips to create automatic transitions. Access lots of fun presets and options by right-clicking on clips."), "arrow":True},
-                                     {"id":"3", "x":200, "y":100, "object_id":"dockVideoContents", "text":_("<b>Video Preview:</b> Watch your timeline video preview here. Use the buttons (play, rewind, fast-forward) to control the video playback."), "arrow":True},
-                                     {"id":"4", "x":20, "y":-35, "object_id":"propertyTableView", "text":_("<b>Properties:</b> View and change advanced properties of clips and effects here. Right-clicking on clips is usually faster than manually changing properties."), "arrow":True},
-                                     {"id":"5", "x":20, "y":10, "object_id":"transitionsTreeView", "text":_("<b>Transitions:</b> Create a gradual fade from one clip to another. Drag and drop a transition onto the timeline and position it on top of a clip (usually at the beginning or ending)."), "arrow":True},
-                                     {"id":"6", "x":20, "y":20, "object_id":"effectsTreeView", "text":_("<b>Effects:</b> Adjust brightness, contrast, saturation, and add exciting special effects. Drag and drop an effect onto the timeline and position it on top of a clip (or track)"), "arrow":True},
-                                     {"id":"7", "x":-265, "y":-22, "object_id":"export_button", "text":_("<b>Export Video:</b> When you are ready to create your finished video, click this button to export your timeline as a single video file."), "arrow":True}
-                                ]
+        self.tutorial_objects = [
+            {"id": "0",
+             "x": 0,
+             "y": 0,
+             "object_id": "dockVideo",
+             "text": _("<b>Welcome!</b> OpenShot Video Editor is an award-winning, open-source video editing application! This tutorial will walk you through the basics.<br><br>Would you like to automatically send errors and metrics to help improve OpenShot?"),
+             "arrow": False
+             },
+            {"id": "1",
+             "x": 0,
+             "y": 0,
+             "object_id": "filesView",
+             "text": _("<b>Project Files:</b> Get started with your project by adding video, audio, and image files here. Drag and drop files from your file system."),
+             "arrow": True
+             },
+            {"id": "2",
+             "x": 0,
+             "y": 0,
+             "object_id": "timeline",
+             "text": _("<b>Timeline:</b> Arrange your clips on the timeline here. Overlap clips to create automatic transitions. Access lots of fun presets and options by right-clicking on clips."),
+             "arrow": True
+             },
+            {"id": "3",
+             "x": 10,
+             "y": -27,
+             "object_id": "actionPlay",
+             "text": _("<b>Video Preview:</b> Watch your timeline video preview here. Use the buttons (play, rewind, fast-forward) to control the video playback."),
+             "arrow": True},
+            {"id": "4",
+             "x": 0,
+             "y": 0,
+             "object_id": "propertyTableView",
+             "text": _("<b>Properties:</b> View and change advanced properties of clips and effects here. Right-clicking on clips is usually faster than manually changing properties."),
+             "arrow": True
+             },
+            {"id": "5",
+             "x": 0,
+             "y": 0,
+             "object_id": "transitionsView",
+             "text": _("<b>Transitions:</b> Create a gradual fade from one clip to another. Drag and drop a transition onto the timeline and position it on top of a clip (usually at the beginning or ending)."),
+             "arrow": True
+             },
+            {"id": "6",
+             "x": 0,
+             "y": 0,
+             "object_id": "effectsView",
+             "text": _("<b>Effects:</b> Adjust brightness, contrast, saturation, and add exciting special effects. Drag and drop an effect onto the timeline and position it on top of a clip (or track)"),
+             "arrow": True
+             },
+            {"id": "8",
+             "x": 0,
+             "y": 0,
+             "object_id": "emojisView",
+             "text": _("<b>Emojis:</b> Add exciting and colorful emojis to your project! Drag and drop an emoji onto the timeline. The emoji will become a new Clip when dropped on the Timeline."),
+             "arrow": True
+             },
+            {"id": "7",
+             "x": 10,
+             "y": -27,
+             "object_id": "export_button",
+             "text": _("<b>Export Video:</b> When you are ready to create your finished video, click this button to export your timeline as a single video file."),
+             "arrow": True
+             }
+        ]
 
         # Configure tutorial frame
-        self.dock.setTitleBarWidget(QWidget()) # Prevents window decoration
+        self.dock.setTitleBarWidget(QWidget())  # Prevents window decoration
         self.dock.setAttribute(Qt.WA_NoSystemBackground, True)
         self.dock.setAttribute(Qt.WA_TranslucentBackground, True)
         self.dock.setWindowFlags(Qt.FramelessWindowHint)
         self.dock.setFloating(True)
 
-
         # Connect to interface dock widgets
         self.win.dockFiles.visibilityChanged.connect(functools.partial(self.process, "dockFiles"))
         self.win.dockTransitions.visibilityChanged.connect(functools.partial(self.process, "dockTransitions"))
         self.win.dockEffects.visibilityChanged.connect(functools.partial(self.process, "dockEffects"))
         self.win.dockProperties.visibilityChanged.connect(functools.partial(self.process, "dockProperties"))
         self.win.dockVideo.visibilityChanged.connect(functools.partial(self.process, "dockVideo"))
+        self.win.dockEmojis.visibilityChanged.connect(functools.partial(self.process, "dockEmojis"))
 
         # Process tutorials (1 by 1)
         if self.tutorial_enabled:
